//=============================================================================
//  DWParent: Parent Class of Dynamic Weather Emitters
//  Contains all the necessary functions for the children... bless 'em
//  By Mark 'MarZer' Gillard, props to DJPaul & Wormbo!
//=============================================================================
class DWParent extends Emitter
		abstract;
		
//==================
//  Replication
//==================		
replication
{
    reliable if ( Role == ROLE_Authority )
		CosWave, Charge, ColourMap, RGBToHLS, ReturnFadeColour, RandomColour;
}		
		
//================
//  Operators
//================
static final operator(18) int : (int A, int B)
{
  return (A + B) / 2;
}

static final operator(16) color / ( color A, float B )
{
  local Color Result;
  Result.R = A.R / B;
  Result.G = A.G / B;
  Result.B = A.B / B;
  return Result;
}

static final operator(22) color Mix ( color A, color B )
{
  local color Result;
  Result.R = A.R : B.R;
  Result.G = A.G : B.G;
  Result.B = A.B : B.B;
  return Result;
}

//==============================
//  General Maths Functions
//==============================
simulated function float CosWave (float M, float N, float X, float T, bool Inverse) 
{
	local float Y;
	if (Inverse == false)
	{
		Y = (((M - N) * Cos(2 * Pi * (1 / (2 * T)) * X)) / 2) + (M : N);
	}
	else
	{
		Y = (((M - N) * -Cos(2 * Pi * (1 / (2 * T)) * X)) / 2) + (M : N);
	}
	return Y;
}

simulated function float Charge (float I, float C, bool ChargingUp)
{
	local float Y;
	if (ChargingUp == true)
	{
		Y = C + I;
	}
	else
	{
		Y = C - I;
	}
	return Y;
}

//=======================
//  Colour Functions
//=======================
simulated function vector ColourMap ( vector rgb)
 {
  local float min;
  local float max;
  local vector hls;
  local float r,g,b,h,l,s;

  rgb.x= Fclamp(rgb.x,0,1);
  rgb.y= Fclamp(rgb.y,0,1);
  rgb.z= Fclamp(rgb.z,0,1);

  r=rgb.x;
  g=rgb.y;
  b=rgb.z;
 
  max = Fmax(fmax(r,g),b);
  min = Fmin(Fmin(r,g),b);

  l = (max+min)/2;

  if (max==min)
   {
     s = 0;
     h = 0;
   }
  else
   {
    if (l < 0.5)  s =(max-min)/(max+min);
    If (l >=0.5)  s =(max-min)/(2.0-max-min);
   }
  
  If (R == max)  h  = (G-B)/(max-min);
  If (G == max) h = 2.0 + (B-R)/(max-min);
  If (B == max)    h = 4.0 + (R-G)/(max-min);

  hls.x=(h/6)*255;
  hls.y=(l*255);
  hls.z=(s*127);

  return( hls);
}

simulated function RGBToHLS(color InRGB, out byte Hue, out byte Luminance, out byte Saturation)
{
    local vector RGB, HLS;
    
    RGB.X = InRGB.R / 255.0;
    RGB.Y = InRGB.G / 255.0;
    RGB.Z = InRGB.B / 255.0;
    HLS = colourmap(RGB);
    Hue = HLS.X;
    Luminance = HLS.Y;
    Saturation = HLS.Z;
}
		
simulated function color ReturnFadeColour(color MinColour, color MaxColour, float X, float T, bool Inverse)
{
	local color ReturnCol;
	local float Y;
	
	if(Inverse == false)
	{
		Y = ((Cos(2 * Pi * (1 / (2 * T)) * X)) / 2) + 0.5;
	}
	else
	{
		Y = ((-Cos(2 * Pi * (1 / (2 * T)) * X)) / 2) + 0.5;
	}
	
	ReturnCol.R = float(MaxColour.R - MinColour.R) * Y;
	ReturnCol.G = float(MaxColour.G - MinColour.G) * Y;
	ReturnCol.B = float(MaxColour.B - MinColour.B) * Y;
	Return MinColour + ReturnCol;
}

simulated function color RandomColour(color MinColour, color MaxColour)
{
	local color ReturnCol;
	local float Y;
	Y = RandRange(0, 1);
	ReturnCol.R = float(MaxColour.R - MinColour.R) * Y;
	ReturnCol.G = float(MaxColour.G - MinColour.G) * Y;
	ReturnCol.B = float(MaxColour.B - MinColour.B) * Y;
	Return MinColour + ReturnCol;
}

defaultproperties
{
	  bDynamicLight=true
	  LightCone=128
	  LightRadius=150
	  LightPeriod=32
      bAlwaysRelevant=true
      bNoDelete=true
	  bStatic=false
	  RemoteRole=ROLE_SimulatedProxy
}